(require 'cl)


(defvar cleite:dialog:mode-map
  (let ((map (make-keymap)))
	(define-key map "q" 'cleite:dialog:quit)
	map))

(defvar cleite:dialog:mode-hook nil)

(defun cleite:dialog:mode ()
  (interactive)
  (kill-all-local-variables)
  (use-local-map cleite:dialog:mode-map)
  (setq mode-name "Cleite Dialog")
  (setq major-mode 'cleite:dialog:mode)
  (make-local-variable 'cleite:dialog:parent-window)
  (run-hooks 'cleite:dialog:mode-hook))

(defun cleite:dialog:quit ()
  (interactive)
  (let ((buffer        (current-buffer))
		(parent-window cleite:dialog:parent-window))
	
	(delete-windows-on buffer)
	(kill-buffer       buffer)

	(when (window-live-p parent-window)
	  (select-window parent-window))))
  

(defmacro cleite:with-dialog-buffer (&rest rest)
  (declare (indent 0))
  (let ((buffer        (gensym))
		(window        (gensym))
		(parent-window (gensym)))
	
	`(let ((,parent-window (selected-window)))

	   (let ((,buffer (generate-new-buffer "*dialog*"))
			 (,window (split-window-vertically)))
	   
		 (select-window    ,window)
		 (switch-to-buffer ,buffer)
		 (cleite:dialog:mode)
		 (setq fill-column (window-width))
		 (setq cleite:dialog:parent-window ,parent-window)
		 (setq buffer-read-only nil)
		 (erase-buffer)
		 
		 ,@rest

		 (setq buffer-read-only t)))))


(provide 'cleite-dialog)
